@@ -5,11 +5,15 @@ package Padre::Config::Style;
 use 5.008;
 use strict;
 use warnings;
-use Carp ();
-use Params::Util qw{ _IDENTIFIER _HASH };
-use YAML::Tiny ();
+use Carp         ();
+use Params::Util ();
+use YAML::Tiny   ();
+
+our $VERSION = '0.63';
+
+
+
 
-our $VERSION = '0.50';
 
 ######################################################################
 # Constructor
@@ -17,10 +21,10 @@ our $VERSION = '0.50';
 sub new {
 	my $class = shift;
 	my $self = bless {@_}, $class;
-	unless ( _IDENTIFIER( $self->name ) ) {
+	unless ( Params::Util::_IDENTIFIER( $self->name ) ) {
 		Carp::croak("Missing or invalid style name");
 	}
-	unless ( _HASH( $self->data ) ) {
+	unless ( Params::Util::_HASH( $self->data ) ) {
 		Carp::croak("Missing or invalid style data");
 	}
 	return $self;
@@ -58,7 +62,7 @@ sub data {
 
 1;
 
-# Copyright 2008-2009 The Padre development team as listed in Padre.pm.
+# Copyright 2008-2010 The Padre development team as listed in Padre.pm.
 # LICENSE
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl 5 itself.
