@@ -23,7 +23,7 @@ use File::Basename ();
 use Padre::Wx      ();
 use Padre::DB      ();
 
-our $VERSION = '0.50';
+our $VERSION = '0.63';
 
 #####################################################################
 # Document Registration
@@ -41,10 +41,10 @@ sub _initialize {
 	return if %EXT_BINARY; # call it only once
 
 	%EXT_BINARY = map { $_ => 1 } qw{
-		aiff  au    avi  bmp  cache  dat   doc  gif  gz   icns
+		aiff  au    avi  bmp  cache  dat   doc  docx gif  gz   icns
 		jar   jpeg  jpg  m4a  mov    mp3   mpg  ogg  pdf  png
 		pnt   ppt   qt   ra   svg    svgz  svn  swf  tar  tgz
-		tif   tiff  wav  xls  xlw    zip
+		tif   tiff  wav  xls  xlw    xlsx  zip
 	};
 
 	# This is the primary file extension to mime-type mapping
@@ -54,16 +54,25 @@ sub _initialize {
 		asm   => 'text/x-asm',
 		bat   => 'text/x-bat',
 		bml   => 'text/x-bml',            # dreamwidth file format
+		c     => 'text/x-c',
+		cc    => 'text/x-c++src',
 		cpp   => 'text/x-c++src',
+		cxx   => 'text/x-c++src',
+		'c++' => 'text/x-c++src',
+		hh    => 'text/x-c++src',
+		hpp   => 'text/x-c++src',
+		hxx   => 'text/x-c++src',
+		'h++' => 'text/x-c++src',
 		css   => 'text/css',
 		diff  => 'text/x-patch',
 		e     => 'text/x-eiffel',
 		f     => 'text/x-fortran',
 		htm   => 'text/html',
 		html  => 'text/html',
+		i     => 'text/x-c',              # C code that should not be preprocessed
+		ii    => 'text/x-c++src',         # C++ code that should not be preprocessed
 		js    => 'application/javascript',
 		json  => 'application/json',
-		latex => 'application/x-latex',
 		lsp   => 'application/x-lisp',
 		lua   => 'text/x-lua',
 		mak   => 'text/x-makefile',
@@ -81,7 +90,14 @@ sub _initialize {
 		plx   => \&perl_mime_type,
 		pm    => \&perl_mime_type,
 		pod   => \&perl_mime_type,
+		psgi  => 'application/x-psgi',
+		sty   => 'application/x-latex',
 		t     => \&perl_mime_type,
+		tex   => 'application/x-latex',
+
+		# Lacking a better solution, define our own MIME
+		xs => 'text/x-perlxs',
+		tt => 'text/x-perltt',
 
 		# Compiled Perl Module or gimme5's output
 		pmc   => \&perl_mime_type,
@@ -110,10 +126,9 @@ sub _initialize {
 	# to confirm that the MIME type is either the official type, or the primary
 	# one in use by the relevant language community.
 
-	# name => Human readable name
+	# name  => Human readable name
 	# lexer => The Scintilla lexer to be used
 	# class => document class
-
 	%MIME_TYPES = (
 		'text/x-abc' => {
 			name  => 'ABC',
@@ -139,8 +154,13 @@ sub _initialize {
 			lexer => Wx::wxSTC_LEX_NULL,  #
 		},
 
+		'text/x-c' => {
+			name  => 'C',
+			lexer => Wx::wxSTC_LEX_CPP,
+		},
+
 		'text/x-c++src' => {
-			name  => 'c++',
+			name  => 'C++',
 			lexer => Wx::wxSTC_LEX_CPP,   # CONFIRMED
 		},
 		'text/css' => {
@@ -168,7 +188,7 @@ sub _initialize {
 			lexer => Wx::wxSTC_LEX_HTML,    # CONFIRMED
 		},
 		'application/javascript' => {
-			name  => 'Javascript',
+			name  => 'JavaScript',
 			lexer => Wx::wxSTC_LEX_ESCRIPT, # CONFIRMED
 		},
 		'application/json' => {
@@ -176,7 +196,7 @@ sub _initialize {
 			lexer => Wx::wxSTC_LEX_ESCRIPT, # CONFIRMED
 		},
 		'application/x-latex' => {
-			name  => 'Latex',
+			name  => 'LaTeX',
 			lexer => Wx::wxSTC_LEX_LATEX,   # CONFIRMED
 		},
 		'application/x-lisp' => {
@@ -184,7 +204,7 @@ sub _initialize {
 			lexer => Wx::wxSTC_LEX_LISP,    # CONFIRMED
 		},
 		'application/x-shellscript' => {
-			name  => 'Shellscript',
+			name  => Wx::gettext('Shell Script'),
 			lexer => Wx::wxSTC_LEX_BASH,
 		},
 		'text/x-lua' => {
@@ -208,6 +228,10 @@ sub _initialize {
 			lexer => Wx::wxSTC_LEX_PERL,     # CONFIRMED
 			class => 'Padre::Document::Perl',
 		},
+		'application/x-psgi' => {
+			name  => 'PSGI',
+			lexer => Wx::wxSTC_LEX_PERL,     # CONFIRMED
+		},
 		'text/x-python' => {
 			name  => 'Python',
 			lexer => Wx::wxSTC_LEX_PYTHON,   # CONFIRMED
@@ -236,44 +260,53 @@ sub _initialize {
 		'text/x-config' => {
 			name  => 'Config',
 			lexer => Wx::wxSTC_LEX_CONF,
+			class => 'Padre::Document::Config',
 		},
 
 		# text/xml specifically means "human-readable XML".
 		# This is prefered to the more generic application/xml
 		'text/xml' => {
 			name  => 'XML',
-			lexer => Wx::wxSTC_LEX_XML,       # CONFIRMED
+			lexer => Wx::wxSTC_LEX_XML, # CONFIRMED
 		},
 
 		'text/x-yaml' => {
 			name  => 'YAML',
-			lexer => Wx::wxSTC_LEX_YAML,      # CONFIRMED
+			lexer => Wx::wxSTC_LEX_YAML, # CONFIRMED
 		},
 		'application/x-pir' => {
 			name  => 'PIR',
-			lexer => Wx::wxSTC_LEX_NULL,      # CONFIRMED
+			lexer => Wx::wxSTC_LEX_NULL, # CONFIRMED
 		},
 		'application/x-pasm' => {
 			name  => 'PASM',
-			lexer => Wx::wxSTC_LEX_NULL,      # CONFIRMED
+			lexer => Wx::wxSTC_LEX_NULL, # CONFIRMED
 		},
 		'application/x-perl6' => {
 			name  => 'Perl 6',
-			lexer => Wx::wxSTC_LEX_NULL,      # CONFIRMED
+			lexer => Wx::wxSTC_LEX_NULL, # CONFIRMED
 		},
 		'text/plain' => {
-			name  => 'Text',
-			lexer => Wx::wxSTC_LEX_NULL,      # CONFIRMED
+			name  => Wx::gettext('Text'),
+			lexer => Wx::wxSTC_LEX_NULL, # CONFIRMED
+		},
+
+		# Completely custom mime types
+		'text/x-perlxs' => {             # totally not confirmed
+			name  => 'XS',
+			lexer => Wx::wxSTC_LEX_CPP,  # for the lack of a better XS lexer (vim?)
+		},
+		'text/x-perltt' => {
+			name  => 'Template Toolkit',
+			lexer => Wx::wxSTC_LEX_HTML,
 		},
 	);
 
-	# TODO:
+	# TO DO:
 	# add some mime-type for pod files
 	# or remove the whole Padre::Document::POD class as it is not in use
 	#'text/x-pod'         => 'Padre::Document::POD',
 
-
-
 	# array ref of objects with value and mime_type fields that have the raw values
 	__PACKAGE__->read_current_highlighters_from_db();
 
@@ -304,7 +337,7 @@ sub get_lexer {
 	return $MIME_TYPES{$mime_type}{lexer};
 }
 
-# TODO: Set some reasonable default highlighers for each mime-type for when there
+# TO DO: Set some reasonable default highlighers for each mime-type for when there
 # are no plugins. e.g. For Perl 6 style files that should be plain text.
 # Either allow the plugins to set the defaults (maybe allow the plugin that implements
 # the special features of this mime-type to pick the default or shall we have a list of
@@ -317,14 +350,14 @@ sub add_mime_class {
 	my $class = shift;
 	if ( not $MIME_TYPES{$mime} ) {
 
-		# TODO: display on the GUI
+		# TO DO: display on the GUI
 		warn "Mime type $mime is not supported when add_mime_class($class) was called\n";
 		return;
 	}
 
 	if ( $MIME_TYPES{$mime}{class} ) {
 
-		# TODO: display on the GUI
+		# TO DO: display on the GUI
 		warn "Mime type $mime already has a class '$MIME_TYPES{$mime}{class}' when add_mime_class($class) was called\n";
 		return;
 	}
@@ -337,14 +370,14 @@ sub remove_mime_class {
 
 	if ( not $MIME_TYPES{$mime} ) {
 
-		# TODO: display on GUI
+		# TO DO: display on GUI
 		warn "Mime type $mime is not supported when remove_mime_class($mime) was called\n";
 		return;
 	}
 
 	if ( not $MIME_TYPES{$mime}{class} ) {
 
-		# TODO: display on GUI
+		# TO DO: display on GUI
 		warn "Mime type $mime does not have a class entry when remove_mime_class($mime) was called\n";
 		return;
 	}
@@ -357,7 +390,7 @@ sub get_mime_class {
 
 	if ( not $MIME_TYPES{$mime} ) {
 
-		# TODO: display on GUI
+		# TO DO: display on GUI
 		warn "Mime type $mime is not supported when remove_mime_class($mime) was called\n";
 		return;
 	}
@@ -398,7 +431,7 @@ sub get_highlighter_name {
 	my $self        = shift;
 	my $highlighter = shift;
 
-	# TODO this can happen if the user configureda highlighter but on the next start
+	# TO DO this can happen if the user configureda highlighter but on the next start
 	# the highlighter is not available any more
 	# we need to handle this situation
 	return '' if !defined($highlighter);
@@ -434,7 +467,7 @@ sub read_current_highlighters_from_db {
 		$MIME_TYPES{$mime_type}{current_highlighter} = 'stc';
 	}
 
-	# TODO check if the highlighter is really available
+	# TO DO check if the highlighter is really available
 	foreach my $e (@$current_highlighters) {
 		$MIME_TYPES{ $e->mime_type }{current_highlighter} = $e->value;
 
@@ -476,7 +509,7 @@ sub add_highlighter_to_mime_type {
 	my $self   = shift;
 	my $mime   = shift;
 	my $module = shift; # module name or stc to indicate Scintilla
-	                    # TODO check overwrite, check if it is listed in HIGHLIGHTER_EXPLANATIONS
+	                    # TO DO check overwrite, check if it is listed in HIGHLIGHTER_EXPLANATIONS
 	$MIME_TYPES{$mime}{highlighters}{$module} = 1;
 }
 
@@ -485,7 +518,7 @@ sub remove_highlighter_from_mime_type {
 	my $mime   = shift;
 	my $module = shift;
 
-	# TODO check overwrite
+	# TO DO check overwrite
 	delete $MIME_TYPES{$mime}{highlighters}{$module};
 }
 
@@ -585,35 +618,104 @@ sub guess_mimetype {
 	# Fall back on deriving the type from the content.
 	# Hardcode this for now for the cases that we care about and
 	# are obvious.
-	if ( $text and $text =~ /\A#!/m ) {
+	if ( defined $text ) {
+		my $eval_mime_type = eval {
 
-		# Found a hash bang line
-		if ( $text =~ /\A#![^\n]*\bperl6?\b/m ) {
-			return $self->perl_mime_type($text);
-		}
-		if ( $text =~ /\A---/ ) {
-			return 'text/x-yaml';
-		}
-	}
+			# Is this a script of some kind?
+			if ( $text =~ /\A#!/m ) {
+				if ( $text =~ /\A#![^\n]*\bperl6?\b/m ) {
+					return $self->perl_mime_type($text);
+				}
+				if ( $text =~ /\A#![^\n]*\b(?:z|k|ba)?sh\b/ ) {
+					return 'application/x-shellscript';
+				}
+			}
 
-	# Try to identify Perl Scripts based on soft criterias as a last resort
-	# TODO: Improve the tests
-	if ( defined($text) ) {
-		my $Score = 0;
-		if ( $text =~ /(use \w+\:\:\w+.+?\;[\r\n][\r\n.]*){3,}/ )           { $Score += 2; }
-		if ( $text =~ /use \w+\:\:\w+.+?\;[\r\n]/ )                         { $Score += 1; }
-		if ( $text =~ /require ([\"\'])[a-zA-Z0-9\.\-\_]+\1\;[\r\n]/ )      { $Score += 1; }
-		if ( $text =~ /[\r\n]sub \w+ ?(\(\$*\))? ?\{([\s\t]+\#.+)?[\r\n]/ ) { $Score += 1; }
-		if ( $text =~ /\=\~ ?[sm]?\// )                                     { $Score += 1; }
-		if ( $text =~ /\bmy [\$\%\@]/ )                                     { $Score += .5; }
-		if ( $text =~ /1\;[\r\n]+$/ )                                       { $Score += .5; }
-		if ( $text =~ /\$\w+\{/ )                                           { $Score += .5; }
-		if ( $text =~ /\bsplit[ \(]\// )                                    { $Score += .5; }
-		return $self->perl_mime_type($text) if $Score >= 3;
+			# YAML will start with a ---
+			if ( $text =~ /\A---/ ) {
+				return 'text/x-yaml';
+			}
+
+			# Try to identify Perl Scripts based on soft criterias as a last resort
+			# TO DO: Improve the tests
+			SCOPE: {
+				my $score = 0;
+				if ( $text =~ /(use \w+\:\:\w+.+?\;[\r\n][\r\n.]*){3,}/ )           { $score += 2; }
+				if ( $text =~ /use \w+\:\:\w+.+?\;[\r\n]/ )                         { $score += 1; }
+				if ( $text =~ /require ([\"\'])[a-zA-Z0-9\.\-\_]+\1\;[\r\n]/ )      { $score += 1; }
+				if ( $text =~ /[\r\n]sub \w+ ?(\(\$*\))? ?\{([\s\t]+\#.+)?[\r\n]/ ) { $score += 1; }
+				if ( $text =~ /\=\~ ?[sm]?\// )                                     { $score += 1; }
+				if ( $text =~ /\bmy [\$\%\@]/ )                                     { $score += .5; }
+				if ( $text =~ /1\;[\r\n]+$/ )                                       { $score += .5; }
+				if ( $text =~ /\$\w+\{/ )                                           { $score += .5; }
+				if ( $text =~ /\bsplit[ \(]\// )                                    { $score += .5; }
+				return $self->perl_mime_type($text) if $score >= 3;
+			}
+
+			# Look for Template::Toolkit syntax
+			#  - traditional syntax:
+			return 'text/x-perltt'
+				if $text =~ /\[\%[\+\-\=\~]? (PROCESS|WRAPPER|FOREACH|BLOCK|END|INSERT|INCLUDE)\b .* [\+\-\=\~]?\%\]/;
+
+			#  - default alternate styles (match 2 tags)
+			return 'text/x-perltt'
+				if $text
+					=~ /(\%\%[\+\-\=\~]? (PROCESS|WRAPPER|FOREACH|BLOCK|END|INSERT|INCLUDE)\b .* [\+\-\=\~]?\%\%.*){2}/s;
+			return 'text/x-perltt'
+				if $text
+					=~ /(\[\*[\+\-\=\~]? (PROCESS|WRAPPER|FOREACH|BLOCK|END|INSERT|INCLUDE)\b .* [\+\-\=\~]?\*\].*){2}/s;
+
+			#  - other languages defaults (match 3 tags)
+			return 'text/x-perltt'
+				if $text
+					=~ /(\<([\?\%])[\+\-\=\~]? (PROCESS|WRAPPER|FOREACH|BLOCK|END|INSERT|INCLUDE)\b .* [\+\-\=\~]?\1\>.*){3}/s;
+			return 'text/x-perltt'
+				if $text
+					=~ /(\<\%[\+\-\=\~]? (PROCESS|WRAPPER|FOREACH|BLOCK|END|INSERT|INCLUDE)\b .* [\+\-\=\~]?\>.*){3}/s;
+			return 'text/x-perltt'
+				if $text
+					=~ /(\<\!\-\-[\+\-\=\~]? (PROCESS|WRAPPER|FOREACH|BLOCK|END|INSERT|INCLUDE)\b .* [\+\-\=\~]?\-\-\>.*){3}/s;
+
+			#  - traditional, but lowercase syntax (3 tags)
+			return 'text/x-perltt'
+				if $text
+					=~ /(\[\%[\+\-\=\~]? (PROCESS|WRAPPER|FOREACH|BLOCK|END|INSERT|INCLUDE)\b .* [\+\-\=\~]?\%\].*){3}/si;
+
+			# Look for HTML (now we can be relatively confident it's not HTML inside Perl)
+			if ( $text =~ /\<\/(?:html|body|div|p|table)\>/ ) {
+
+				# Is it Template Toolkit HTML?
+				# Only try to text the default [% %]
+				if ( $text =~ /\[\%\-?\s+\w+(?:\.\w+)*\s+\-?\%\]/ ) {
+					return 'text/x-perltt';
+				}
+
+				return 'text/html';
+			}
+
+			# Try to detect plain CSS without HTML around it
+			return 'text/css'
+				if ( $text !~ /\<\w+\/?\>/ )
+				and ( $text =~ /^([\.\#]?\w+( [\.\#]?\w+)*)(\,[\s\t\r\n]*([\.\#]?\w+( [\.\#]?\w+)*))*[\s\t\r\n]*\{/ );
+
+			# LUA detection
+			my $lua_score = 0;
+			for ( 'end', 'it', 'in', 'nil', 'repeat', '...', '~=' ) {
+				$lua_score += 1.1 if $text =~ /[\s\t]$_[\s\t]/;
+			}
+			$lua_score += 2.01 if $text =~ /^[\s\t]?function[\s\t]+\w+[\s\t]*\([\w\,]*\)[\s\t\r\n]+[^\{]/;
+			$lua_score -= 5.02 if $text =~ /[\{\}]/; # Not used in lua
+			$lua_score += 3.04 if $text =~ /\-\-\[.+?\]\]\-\-/s; # Comment
+			return 'text/x-lua' if $lua_score >= 5;
+
+			return '';
+		};
+		return if $@;
+		return $eval_mime_type if $eval_mime_type;
 	}
 
 	# Fallback mime-type of new files, should be configurable in the GUI
-	# TODO: Make it configurable in the GUI :)
+	# TO DO: Make it configurable in the GUI :)
 	unless ($filename) {
 		return $self->perl_mime_type($text);
 	}
@@ -672,9 +774,20 @@ sub is_perl6 {
 }
 
 
+sub menu_view_mimes {
+	my %menu_view_mimes = ();
+	foreach my $mime_type ( keys %MIME_TYPES ) {
+		my $mime_type_name = $MIME_TYPES{$mime_type}{name};
+		if ($mime_type_name) {
+			$menu_view_mimes{$mime_type_name} = $mime_type;
+		}
+	}
+	return %menu_view_mimes;
+}
+
 1;
 
-# Copyright 2008-2009 The Padre development team as listed in Padre.pm.
+# Copyright 2008-2010 The Padre development team as listed in Padre.pm.
 # LICENSE
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl 5 itself.
