@@ -4,15 +4,14 @@ use 5.008;
 use strict;
 use warnings;
 use File::Copy;
-use File::Spec     ();
-use File::Basename ();
-use Params::Util qw{_INSTANCE};
+use File::Spec      ();
+use File::Basename  ();
 use Padre::Current  ();
 use Padre::Util     ();
 use Padre::Wx       ();
 use Padre::Constant ();
 
-our $VERSION = '0.50';
+our $VERSION = '0.63';
 our @ISA     = 'Wx::TreeCtrl';
 
 # Creates a new Directory Browser object
@@ -149,7 +148,7 @@ sub refresh {
 
 	# Lock the gui here to make the updates look slicker
 	# The locker holds the gui freeze until the update is done.
-	my $locker = $self->main->freezer;
+	my $lock = $self->main->lock('UPDATE');
 
 	# If the project have changed or the project root folder updates or
 	# the search is not in use anymore (was just used)
@@ -319,10 +318,20 @@ sub _updated_dir {
 	my $dir    = shift;
 	my $cached = $self->{CACHED}->{$dir};
 
+	my $file = $self->parent->{file};
+	my $mtime;
+
+	if ( defined($file) ) {
+		$file->browse_mtime($dir);
+	} else {
+		$mtime = ( stat($dir) )[10];
+	}
+
 	if (   not defined $cached
 		or !$cached->{Data}
 		or !$cached->{Change}
-		or ( stat $dir )[10] != $cached->{Change}
+		or !defined($mtime)
+		or $mtime != $cached->{Change}
 		or $self->search->{just_used}->{$dir} )
 	{
 		return 1;
@@ -341,7 +350,7 @@ sub _update_subdirs {
 	my $cookie;
 
 	# Loops thought the node's total children
-	for my $item ( 1 .. $self->GetChildrenCount($root) ) {
+	foreach my $item ( 1 .. $self->GetChildrenCount($root) ) {
 
 		( my $node, $cookie ) = $item == 1 ? $self->GetFirstChild($root) : $self->GetNextChild( $root, $cookie );
 		next if not $node->IsOk;
@@ -414,10 +423,11 @@ sub _rename_or_move {
 		my $separator = File::Spec->catfile( '', '' );
 
 		# Moves all cached data of the node and above it to the new path
-		map {
-			$cached->{ $new_file . ( defined $1 ? $1 : '' ) } = $cached->{$_}, delete $cached->{$_}
-				if $_ =~ /^$old_file($separator.+?)?$/
-		} keys %$cached;
+		foreach ( keys %$cached ) {
+			next unless /^$old_file($separator.+?)?$/;
+			$cached->{ $new_file . ( defined $1 ? $1 : '' ) } = $cached->{$_};
+			delete $cached->{$_};
+		}
 
 		$self->{select_item} = 1;
 
@@ -725,12 +735,9 @@ sub _on_tree_item_menu {
 
 	Wx::Event::EVT_MENU(
 		$self,
-		$menu->Append( -1, Wx::gettext('Open In File Browser') ),
+		$menu->Append( -1, Wx::gettext('Open in File Browser') ),
 		sub {
-
-			#Open the current node in file browser
-			require Padre::Wx::Directory::OpenInFileBrowserAction;
-			Padre::Wx::Directory::OpenInFileBrowserAction->new->open_in_file_browser($selected_path);
+			$_[0]->main->on_open_in_file_browser($selected_path);
 		}
 	);
 
@@ -815,7 +822,7 @@ sub _on_tree_item_menu {
 			$self, $pod,
 			sub {
 
-				# TODO Fix this wasting of objects (cf. Padre::Wx::Menu::Help)
+				# TO DO Fix this wasting of objects (cf. Padre::Wx::Menu::Help)
 				require Padre::Wx::DocBrowser;
 				my $help = Padre::Wx::DocBrowser->new;
 				$help->help( $node_data->{name} );
@@ -836,7 +843,7 @@ sub _on_tree_item_menu {
 		$applies_to_node = $self->GetParent($node);
 	}
 
-	my $cached = \%{ $self->{CACHED}->{$applies_to_path} } if defined($applies_to_path);
+	my $cached = defined($applies_to_path) ? \%{ $self->{CACHED}->{$applies_to_path} } : undef;
 	my $show = $cached->{ShowHidden};
 	$hiddenFiles->Check($show);
 	Wx::Event::EVT_MENU(
@@ -867,7 +874,7 @@ sub _on_tree_item_menu {
 
 1;
 
-# Copyright 2008-2009 The Padre development team as listed in Padre.pm.
+# Copyright 2008-2010 The Padre development team as listed in Padre.pm.
 # LICENSE
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl 5 itself.
