@@ -3,13 +3,12 @@ package Wx::Perl::Dialog::Frame;
 use 5.008;
 use strict;
 use warnings;
-use File::Spec ();
-use Wx::Perl::Dialog;
-use Wx::STC ();
+use File::Spec       ();
+use Wx::Perl::Dialog ();
+use Wx::STC          ();
 
-use base 'Wx::Frame';
-
-our $VERSION = '0.50';
+our $VERSION = '0.63';
+our @ISA     = 'Wx::Frame';
 
 sub new {
 	my ($class) = @_;
@@ -22,7 +21,7 @@ sub new {
 		Wx::wxDefaultSize,
 	);
 
-	#    Wx::Event:EVT_ACTIVATE($self, \&on_activate);
+	# Wx::Event:EVT_ACTIVATE($self, \&on_activate);
 	Wx::Event::EVT_CLOSE(
 		$self,
 		sub {
@@ -45,7 +44,7 @@ sub on_activate {
 
 1;
 
-# Copyright 2008-2009 The Padre development team as listed in Padre.pm.
+# Copyright 2008-2010 The Padre development team as listed in Padre.pm.
 # LICENSE
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl 5 itself.
